#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from psychopy import visual, core, gui, data, event, parallel
import time, datetime, os.path, sys
import numpy as np
import sounddevice as sd
home = os.path.expanduser('~')
#wd = home + '\\Box Sync\\AAN4\\Python'
wd = home + '\\desktop\\AAN4'
sys.path.append(wd + '\\defs')
import mn
from AAN4_defs import *
from AAN4_instructions import *

# Check for escape
def check4esc():
    #  Checks for escape. If escape is pressed, it terminates the program.
    keys = event.getKeys(keyList = ['escape'])
    if len(keys)>0 and 'escape' in keys:
        core.quit()
# Add stuff at start of log file
def add_log(f, start, end):
    f = open(f,'r+')
    content = f.read()
    f.seek(0, 0)
    f.write(start.rstrip('\r\n') + '\n' + end.rstrip('\r\n') + '\nfp: ' + 
        str(FP) + '\nValnr: ' + str(valnr) + '\nTask: ' + task +  
        '\n\n' + content)
    f.close()
# Send triggers
def send_triggers(t):
    p_port.setData(t)
    core.wait(0.05) # 50 ms
    p_port.setData(255)
# Random key for debug
def random_key():
    sub_key = ['up','down']
    resp = random.sample(sub_key,1)
    return resp
# Pause def --------------------------------------------------------------------
def paus(pauses,t,block,task,gonogoAlt):
    #  Keyword arguments:
    #  pauses -- list of trialnumbers where there should be a pauses (list)
    #  t -- the current trialnumber (int)
    #  block -- list of blocknumbers, ex (0,0,0,1,1,1,2,2,2) (list)
    #  pport -- what trigger to send for pause (int)
    if t in pauses:
        task_inst.setAutoDraw(False)
        inst.setText(pausInfo % ((max(block)+1) - block[t], gonogoAlt[task]))
        inst.draw()
        win.flip()
        event.waitKeys(keyList=['return'])
        win.flip()
        send_triggers(223)
        core.wait(1)
# Subjective response def ------------------------------------------------------
def sub_def(debug):
    if debug == 1: sub_resp = random_key()[0]
    elif debug == 0: sub_resp = event.waitKeys(keyList=['up','down','escape'])[0]
    aware = None
    if sub_resp == 'escape': core.quit()
    elif sub_resp == 'down': aware = 0
    elif sub_resp == 'up': aware = 1
    return sub_resp, aware
# Transform key presses to logs ------------------------------------------------
def response_check(key, gonogo):
    #  Checks if a key was pressed.
    #  Keyword arguments:
    #  key -- containing either a keypress and a time or nothing (list)
    #  return:
    #  time -- nan if not pressed or time of press if pressed
    if len(key) == 0: pressed, time = 0, 'no'
    elif len(key) != 0: pressed, time = 1, key[0][1]
    if gonogo == pressed: aware = 1
    else: aware = 0
    return str(time), pressed, aware
# Staircase def ----------------------------------------------------------------
def staircase():
    fileName = fileDir+'\\AAN4_cal_'+fps+'.txt'
    df = open(fileName, 'w')
    df.write('fp\ttrial\tstair\tlevel\taware\n')
    conditions=[
        {'label':1, 'startVal': -60, 'nUp':1, 'nDown':1, 'minVal' : -200,
        'maxVal' : 0, 'stepSizes':[8,8,4,4,2,2,1], 'nReversals':12, 'stepType':'lin'},
        {'label':2,'startVal': -80, 'nUp':1, 'nDown':1, 'minVal' : -200,
        'maxVal' : 0, 'stepSizes':[8,8,4,4,2,2,1], 'nReversals':12, 'stepType':'lin'},
        {'label':3, 'startVal': -100, 'nUp':1, 'nDown':1, 'minVal' : -200,
        'maxVal' : 0, 'stepSizes':[8,8,4,4,2,2,1], 'nReversals':12, 'stepType':'lin'},
        ]
    stairs = data.MultiStairHandler(stairType='simple', method = 'random',
                                    conditions=conditions, nTrials=10)
    time = core.Clock()
    trial = 0
    inst.text = spaceInfo
    inst.draw()
    win.flip()
    event.waitKeys(keyList=['return'])
    time.reset()
    win.flip()
    core.wait(1)
    for level in stairs:
        stair = level[1]['label']
        trial += 1 # count trialnumber
        sound = create_stimulus(c = 1, f = 1000, t = SDUR, lvl = level[0],
                                fade = FADE, fs = FS)
        fixation.draw()
        win.flip()
        # Play sound -----------------------------------------------------------
        send_triggers(254) # 7 frames = 50 ms
        core.wait(dur1)
        sd.play(sound, FS, mapping = ch[nr_channels])
        core.wait(dur2)
        win.flip() # turn fixation off
        core.wait(0.5)
        inst.text = subInst
        inst.draw()
        win.flip()
        subj_resp, aware = sub_def(debug) # has check4esc
        event.clearEvents()
        win.flip()
        core.wait(0.6)
        # simulation ================================================#
        if debug == 1:
            core.wait(np.random.uniform(low=0.0, high=1, size=None))
            if level[0] < -86: aware = 0
            elif level[0] > -86: aware = 1
        #============================================================#
        stairs.addResponse(aware)
        #'fp\ttrial\tstair\tlevel\taware\n'
        #  i    i      i      f      i
        df.write('%i\t%i\t%i\t%f\t%i\n'%(FP, trial, stair, level[0], aware))
    print("Time : %.1f" % time.getTime())
# Practice def -----------------------------------------------------------------
def validation():
    if task == 'Validation':
        fileName = fileDir+'\\AAN4_val_'+fps+'_'+str(valnr)+'.txt'
        df = open(fileName, 'w')
        start_time = 'Experiment start: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
        df.write('fp\ttrial\tcond\tlevel\tsub_resp\taware\tvalnr\n')
        inst.text = spaceInfo
        inst.draw()
    if task == 'Practice':
        inst.text = pracInst
        inst.draw()
    win.flip()
    time = core.Clock()
    ntp = 50 # number of practice trials
    sub_list = []
    event.waitKeys(keyList=['return']) # press space to start
    time.reset()
    win.flip()
    # 1000 Hz tone
    sound = create_stimulus(c = 1, f = 1000, t = SDUR, lvl = s,
                            fade = FADE, fs = FS)
    core.wait(1)
    for t in range(ntp):
        fixation.draw()
        win.flip()
        # Play sound -----------------------------------------------------------
        send_triggers(254) # 7 frames = 50 ms
        core.wait(dur1)
        sd.play(sound, FS, mapping = ch[nr_channels])
        core.wait(dur2)
        #core.wait(0.2)
        win.flip() # turn fixation off
        core.wait(0.5)
        inst.text = subInst
        inst.draw()
        win.flip()
        sub_resp, aware = sub_def(debug) # has check4esc
        win.flip()
        core.wait(0.6)
        event.clearEvents()
        # simulation ================================================#
        if debug == 1:
            core.wait(np.random.uniform(low=0.0, high=1, size=None))
            if s < -86: aware = 0
            elif s > -86: aware = 1
        #============================================================#
        sub_list += [aware] # store aware 0,1
        if task == 'Validation': # if validation, make log
            #'fp\ttrial\tcond\tlevel\tsub_resp\taware\tvalnr\n'
            # i    i      i      f       s        i      i
            df.write('%i\t%i\t%i\t%f\t%s\t%i\t%i\n'
                %(FP, t+1, 1, s, sub_resp, aware, int(valnr)))
    if task == 'Validation': # if validation, store timestamps and close.
        df.close()
        end_time = 'Experiment end: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
        add_log(fileName, start_time, end_time)
    # give some output
        aware = calc_aware(sub_list)
        print("Time : %.1f" % time.getTime())
        print('Aware: %.1f' % (aware * 100))
# Experiment def ---------------------------------------------------------------
def experiment():
    # Conditions:
    # 0 = critical 1000Hz (calibrated dB)
    # 1 = control (calibrated + 10 dB)
    # 2 = catch (~0 dB)

    fileName = fileDir+'\\AAN4_fp'+fps+'.txt'
    df = open(fileName, 'w')
    start_time = 'Experiment start: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
    df.write('fp\ttrial\tblock\tcond\tlevel\tgonogo\tpressed\tRT\taware\ttrigger\n')

    crit_list = []
    control_list = []
    catch_list = []

    nte = 600 # number of trials experiment
    nbl = 6 # number of blocks
    # go or nogo
    # Blocks and breaks 600 / 6 = 100
    block = []
    pauses = []
    gonogo = []
    for i in range(nbl):
        pauses += [[]]
        pauses[i] = int(i * 100)
        block += [i] * int(nte/nbl)
        gonogo += [abs(i % 2 - FP % 2)] * int(nte/nbl)
    del pauses[0]
    
    # Create trials for experiment
    cond = []
    level = []
    trigger = []
    # makes the trials (10 * 60 = 600)
    for i in range(60):
        order = list(np.random.permutation(range(10)))
        cond += [([0]*8 + [1]*1 + [2]*1)[i] for i in order]
        level += [([s]*8 + [s+10]*1 + [0]*1)[i] for i in order]
        trigger += [([254]*8 + [253]*1 + [251]*1)[i] for i in order]

    time = core.Clock()
    RT = core.Clock()
    inst.text = expInst % gonogoAlt[gonogo[0]]
    inst.draw()
    win.flip()
    event.waitKeys(keyList=['return'])
    win.flip()
    # Make sounds --------------------------------------------------------------
    crit    = create_stimulus(c=1, f=1000, t=SDUR, lvl=s, fade=FADE, fs=FS)
    control = create_stimulus(c=1, f=1000, t=SDUR, lvl=s+10, fade=FADE, fs=FS)
    catch   = create_stimulus(c=2, f=1000, t=SDUR, lvl=-50, fade=FADE, fs=FS)
    sounds = [crit, control, catch]
    # WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM WARM UP SYSTEM
    sd.play(catch, FS, mapping = ch[nr_channels])
    core.wait(1)
    # Trial loop ---------------------------------------------------------------
    trial = 359
    for t in range(nte): # 
        paus(pauses, t, block, gonogo[t], gonogoAlt)
        task_inst.text = blockInst % gonogoAlt[gonogo[t]]
        task_inst.setAutoDraw(True)
        win.flip()
        #diodeL.draw()
        fixation.draw()
        send_triggers(trigger[t]) # 50 ms
        win.flip() # fixation on
        time.reset()
        core.wait(dur1)
        sd.play(sounds[cond[t]], FS, mapping = ch[nr_channels])
        core.wait(dur2)
        #print('500 ms = %.1f ms' % time.getTime()*1000)
        win.flip() # fixation off
        event.clearEvents()
        RT.reset()
        for frame in range(trial):
            win.flip()
        key = event.getKeys(keyList = ['space'], timeStamped = RT)
        RTs, pressed, aware = response_check(key, gonogo[t])
        if cond[t] == 0: # critical
            crit_list += [aware]
        if cond[t] == 1: # control
            control_list += [aware]
        if cond[t] == 2: # catch
            catch_list += [aware]
        check4esc()
        event.clearEvents()
        #print('3000 ms = %.1f ms' % time.getTime()*1000)
        #fp\ttrial\tblock\tcond\tlevel\tgonogo\tpressed\tRT\taware\ttrigger\n
        #i    i       i     i      f      i        i     s     i      i
        df.write('%i\t%i\t%i\t%i\t%f\t%i\t%i\t%s\t%i\t%i\n'
            % (FP, t+1, block[t]+1, cond[t], level[t], gonogo[t], pressed,
               RTs, aware, trigger[t]))
    df.close()
    end_time = 'Experiment end: ' + datetime.datetime.now().strftime("%y-%m-%d %H:%M")
    add_log(fileName, start_time, end_time)
    # give some output to user
    aware_crit = calc_aware(crit_list)
    aware_catch = calc_aware(catch_list)
    aware_control = calc_aware(control_list)
    print('A crit: %.1f\nA catch: %.1f\nA control: %.1f\n' % (aware_crit * 100, aware_catch * 100, aware_control * 100))

# check if there is a memory file
if os.path.isfile('mem.txt'):
    print('Found memory!')
    with open('mem.txt', 'r') as memory:
        memory_data = [data.split(',') for data in memory][0]
    expInfo = gui.Dlg(title="Auditory detection")
    expInfo.addField('Subject', memory_data[0])
    expInfo.addField('Valnr', memory_data[1])
    expInfo.addText('Experiment Info')
    expInfo.addField('Task', choices = ['Validation','Practice','Calibration','Experiment'])
    expInfo.addField('Level', float(memory_data[3]))
    ok_data = expInfo.show() # show dialog and wait for OK or Cancel
    if expInfo.OK:  # or if ok_data is not None
        if ok_data[3] > 0:
            print('Too high level!')
            core.quit()
        print('Subject: %i\nTask: %s\nLevel: %f\n' %(int(ok_data[0]), ok_data[2], ok_data[3]))
    else:
        print('user cancelled')
        core.quit()
else:
    print('No memory, making.')
    # present a dialogue to give subject nr and select program
    expInfo = gui.Dlg(title="Auditory detection")
    expInfo.addField('Subject', '01')
    expInfo.addField('Valnr', '01')
    expInfo.addText('Experiment Info')
    expInfo.addField('Task', choices = ['Validation','Practice','Calibration','Experiment'])
    expInfo.addField('Level', -40)
    ok_data = expInfo.show()  # show dialog and wait for OK or Cancel
    if expInfo.OK:  # or if ok_data is not None
        if ok_data[3] > 0:
            print('Too high level!')
            core.quit()
        print('Subject: %i\nTask: %s\nLevel: %f\n' %(int(ok_data[0]), ok_data[2], ok_data[3]))
    else:
        print('user cancelled')
        core.quit()
#
# Parallel port
#p_port = parallel.ParallelPort(address = "0x4020") # room 225 (right)
p_port = parallel.ParallelPort(address='0x3028') # room 223 (left)
p_port.setData(255)
sd.default.device = "ASIO Fireface USB"
nr_channels = 1 # 0 = 2, 1 = 4
print('Sound device ------------------------------------------------------------')
print(sd.default.device)

# Global variables
fileDir = wd + '\\logs'
FADE = 0.005
FS = 48000
SDUR = 0.1
ch = [[1,2],[3,4,1,2]]

dur1 = 0.17
dur2 = 0.05 #0.585
debug = 0

# from prompt
fps = ok_data[0]
FP = int(ok_data[0])
valnr = ok_data[1]
task = ok_data[2]
s = ok_data[3]

with open('mem.txt', 'w') as memory:
    memory.write(','.join(str(data) for data in [fps, valnr, task, str(s)]))
# Objects
#Screen 1920, 1080
win = visual.Window([1920, 1080], allowGUI=False, fullscr=True,
    monitor='testMonitor', units='height')
#Stimuli
fixation = visual.TextStim(win, color='black', height=0.03, text='+')
task_inst = visual.TextStim(win, color='black', height=0.02, text='', pos=(0, 0.05))
inst = visual.TextStim(win, color='black', height=0.02, text='')

diodeL = visual.Rect(
    win=win, name='diodeL',
    width=(0.025, 0.025)[0], height=(0.025, 0.025)[1],
    ori=0, pos=(-0.88, 0.49),
    lineWidth=1, lineColor=[1.000,1.000,1.000], lineColorSpace='rgb',
    fillColor=[1.000,1.000,1.000], fillColorSpace='rgb',
    opacity=1, depth=0.0, interpolate=True)
#==============================================================================#
# Procedure
if task == 'Experiment':
    experiment()
if task == 'Calibration':
    staircase()
if task == 'Validation':
    validation()
if task == 'Practice':
    validation()

inst.text = endInfo % task
inst.draw()
win.flip()
event.waitKeys(keyList = ['q'])

win.close()
core.quit()