#!/usr/bin/python
# -*- coding: utf-8 -*-

# Function for creating stimulus for Rasmus Eklund's experiment)
# Created: MN-2017-10-10; Revised: RE-2018-10-25

import mn, random
import numpy as np

# AAN defs --------------------------------------------------------------------#
def calc_aware(sub_list):
    '''
    Keyword arguments:
    sub_list -- list of numbers: 0 = unaware, 1 = aware

    Return:
    aware -- (aware/(aware + unaware)) (float)
    '''

    aware_index = []
    # find and enumerate all reported aware (sub_list)
    for i, x in enumerate(sub_list):
        if x == 1:
            aware_index += [i]

    # calculate percent reported aware
    aware = sum(sub_list)*1.0/len(sub_list)

    return aware

def create_stimulus(c = 1, f = 1000, t = 0.1, lvl = 0.0, fade = 0.005, fs = 48000):
    ''' Creates stimulus tones
    Keyword arguments:
    c -- 1 = tone, 2 = catch
    t -- signal duration(s) (float)
    lvl -- signal level, rms dB (float)
    fade -- Duration of fade-in and fade-out, in seconds (float)
    fs -- Sampling frequency (int)

    Return:
    stimulus -- stereo * 2 signal (4xN numpy array)
    '''

    # Create signal ------------------------------------------------------------
    s = mn.create_sinusoid (freq = f, phase = 0, fs = fs, dur = t)
    # --------------------------------------------------------------------------

    # Add zeros to signal to make it as long as the white noise ----------------
    s_stimtracker = mn.set_gain(s, -20) # make signal at higher dB stimtracker
    s = mn.set_gain(s, lvl) # make signal at calibrated lvl
    catch = np.zeros(int(t * fs))
    # --------------------------------------------------------------------------

    # Combine noise and signal into stereo -------------------------------------
    fadesamples = int(fade * fs)
    if (c == 1): # critical or control
        ch1 = mn.fade(s, samples = fadesamples)
        ch2 = mn.fade(s, samples = fadesamples)
        ch3 = mn.fade(s_stimtracker, samples = fadesamples)
        ch4 = mn.fade(s_stimtracker, samples = fadesamples)
    elif (c == 2): # catch
        ch1 = mn.fade(catch, samples = fadesamples)
        ch2 = mn.fade(catch, samples = fadesamples)
        ch3 = mn.fade(catch, samples = fadesamples)
        ch4 = mn.fade(s_stimtracker, samples = fadesamples)


    stimulus = np.transpose(np.array([ch1,ch2,ch3,ch4]))
    #stimulus = np.transpose(np.array([ch1,ch2]))
    # --------------------------------------------------------------------------
    return stimulus